import json
import os.path
from typing import Dict, Callable

from assertpy import assert_that

from deepdriver.sdk.artifact import Artifacts, ArtifactEntry
from deepdriver.sdk.data_types.run import get_run
from deepdriver import logger
from deepdriver import util


# dictionary 형태의 데이터, image, table, chart 등을 log 함수를 통해 서버로 전송
@util.login_required
@util.init_required
def log(data: Dict) -> bool:
    return get_run().log(data)

# image, table, dictionary 형태의 데이터를 log_artifact 를 통해 서버로 전송
@util.login_required
@util.init_required
def upload_artifact(artifact: Artifacts) -> bool:
    return get_run().upload_artifact(artifact)

@util.login_required
@util.init_required
def get_artifact(name: str, type: str, tag: str="", team_name: str="", exp_name: str="") -> Artifacts:
    if team_name =="":
        team_name =get_run().team_name

    if exp_name =="":
        exp_name =get_run().exp_name

    result, result_msg,  artifact_id, artifact_record = get_run().get_artifact(name, type, tag, team_name, exp_name)
    if result == "fail" :
        if result_msg =="not exist" :
            logger.info("artifact is created!")
            return Artifacts(name, type)
        else:
            raise ValueError(result_msg)
    logger.info("artifact is got! \n artifact id :{%d}"%(artifact_id))
    entry_list = []
    for entry in artifact_record.artifact_entry:
        entry_list.append(ArtifactEntry(entry.path, "", entry.size, entry.digest, status="ADD", lfs_yn=entry.lfsYN, repo_tag=entry.repoTag, type=entry.type, metadata=entry.metadata, key=entry.key))
    return Artifacts(name=artifact_record.name,
                     type=artifact_record.type,
                     id=artifact_id,
                     desc=artifact_record.description,
                     meta_data=json.loads(artifact_record.metadata),
                     entry_list=entry_list,
                     exp_name=exp_name
                     )


@util.login_required
@util.init_required
def upload_code(name: str = None, type: str = "CODE", tag: str = "", team_name: str = None, exp_name: str = None,
                path: str = "./", include_func: Callable = lambda path: path.endswith("py") or path.endswith("ipynb"), exclude_func: Callable = lambda path: False) -> None:
    assert_that(name).is_not_none()
    assert_that(path).is_not_none()

    if not team_name :
        team_name = get_run().team_name
    if not exp_name:
        exp_name = get_run().exp_name

    arti = get_artifact(name=name, type=type, tag=tag, team_name=team_name, exp_name=exp_name)

    import glob
    for file_path in glob.glob(os.path.join(path, "**"), recursive=True):
        if os.path.isdir(file_path):
            continue
        if include_func(file_path) and (not exclude_func(file_path)):
            logger.debug(f"path added: {file_path}")
            arti.add(file_path)
    arti.upload()



# Interface.py의 finish()함수를 호출할때 하기의 summary정보를 dictionary 형태로 넘겨준다
@util.init_required
def finish() -> bool:
    from deepdriver import config
    config.update()
    return get_run().finish()


# config 업데이트
@util.login_required
@util.init_required
def update() -> bool:
    from deepdriver import config
    return config.update()

# alert 전송
@util.login_required
@util.init_required
def alert(alert_msg:str) -> bool:
    return get_run().alert(alert_msg)


from assertpy import assert_that
from deepdriver.sdk.interface import interface
# deepdriver 실험환경을 사용하기위한 로그인 과정
# 서버의 login api를 호출하여 key를 서버로 전송하고 결과로서 jwt key를 받는다
def setting(http_host=None, grpc_host=None):
    if http_host is not None:
        interface.set_http_host(http_host)
    if grpc_host is not None:
        interface.set_grpc_host(grpc_host)



import importlib
from typing import Union

import pandas as pd
from assertpy import assert_that

from deepdriver import logger
from deepdriver.sdk.chart.chart import Chart, TYPE_CONFUSION_MATRIX, TYPE_ROC_CURVE, TYPE_HISTOGRAM, TYPE_BAR
from deepdriver.sdk.data_types.image import Image
from deepdriver.sdk.data_types.table import Table


def visualize(obj: Union[Chart, Table, Image]) -> None:
    assert_that(obj).is_not_none()

    def visualize_table(obj):
        # from IPython.core.display import display, HTML
        # display(HTML(obj.data.dataframe._repr_html_()))

        plotly_path = "plotly.graph_objects"
        plotly_module = importlib.import_module(plotly_path)

        fig = plotly_module.Figure(
            data=[plotly_module.Table(
                header=dict(values=obj.data.columns),
                cells=dict(values=obj.data.dataframe.T))
            ],
        )
        fig.show()

    def visualize_image(obj):
        # from IPython.core.display import display, HTML
        # display(HTML(obj.data.dataframe._repr_html_()))

        plotly_path = "plotly.express"
        plotly_module = importlib.import_module(plotly_path)
        import numpy as np
        fig = plotly_module.imshow(
            np.array(obj.data)
        )
        fig.show()

    def visualize_chart_common(obj):
        # line, scatter 차트
        plotly_path = "plotly.express"
        plotly_module = importlib.import_module(plotly_path)
        plotly_chart_func = getattr(plotly_module, obj.chart_type)

        fig = plotly_chart_func(
            x=obj.data.dataframe[obj.data_fields["x"]],
            y=obj.data.dataframe[obj.data_fields["y"]],
            labels=obj.data_fields,
            title=obj.label_fields["title"],
        )
        fig.show()

    def visualize_chart_bar(obj):
        # bar 차트
        plotly_path = "plotly.express"
        plotly_module = importlib.import_module(plotly_path)
        plotly_chart_func = getattr(plotly_module, obj.chart_type)

        fig = plotly_chart_func(
            obj.data.dataframe,
            x=obj.data_fields["x"],
            y=obj.data_fields["y"],
            labels=obj.data_fields,
            title=obj.label_fields["title"],
        )
        fig.show()

    def visualize_chart_histogram(obj):
        # historgram
        plotly_path = "plotly.express"
        plotly_module = importlib.import_module(plotly_path)
        plotly_chart_func = getattr(plotly_module, obj.chart_type)

        if obj.data_fields["y"]:
            fig = plotly_chart_func(
                obj.data.dataframe,
                x=obj.data_fields["x"],
                y=obj.data_fields["y"],
                labels=obj.data_fields,
                title=obj.label_fields["title"],
            )
            fig.show()
        else:
            fig = plotly_chart_func(
                obj.data.dataframe,
                x=obj.data_fields["x"],
                labels=obj.data_fields,
                title=obj.label_fields["title"],
            )
            fig.show()

    def visualize_confusion_matrix(obj):
        # dataframe 에서 x, y, z 값 추출
        actual = obj.data.dataframe['Actual']
        predicted = obj.data.dataframe['Predicted']
        n_redictions = obj.data.dataframe['nPredictions']
        cross_tab_df = pd.crosstab(index=actual, columns=predicted, values=n_redictions, aggfunc=sum)
        logger.debug(f"cross_tab_df : \n{cross_tab_df}")
        """ cross_tab_df
        Predicted  cat  dog  horse
        Actual                    
        cat        5.0  2.0    0.0
        dog        0.0  4.0    1.0
        horse      1.0  0.0    3.0
        """
        labels, z = [], []
        for item in cross_tab_df.transpose().iteritems():
            # item : ('cat', 'list[]')
            labels.append(item[0])
            z.append(list(item[1]))

        # 그래프 그리기
        plotly_path = "plotly.graph_objects"
        plotly_module = importlib.import_module(plotly_path)
        data = plotly_module.Heatmap(z=z, x=labels, y=labels)
        annotations = []
        for i, row in enumerate(z):
            for j, value in enumerate(row):
                annotations.append(
                    {
                        "x": labels[i],
                        "y": labels[j],
                        # "font": {"color": "white"},
                        "text": z[j][i],
                        "xref": "x",
                        "yref": "y",
                        "showarrow": False
                    }
                )
        layout = {
            "title": obj.label_fields["title"],
            "xaxis": {"title": "Predicted"},
            "yaxis": {"title": "Actual"},
            "annotations": annotations
        }
        fig = plotly_module.Figure(data=data, layout=layout)
        fig.show()

    def visualize_roc_curve(obj):
        """
           class  fpr    tpr
        0   cat  0.0  0.000
        1   cat  0.0  0.500
        2   cat  0.0  1.000
        3   cat  1.0  1.000
        4   dog  0.0  0.000
        5   dog  0.0  0.333
        6   dog  0.0  1.000
        7   dog  1.0  1.000
        """
        plotly_path = "plotly.graph_objects"
        plotly_module = importlib.import_module(plotly_path)

        fig = plotly_module.Figure()
        fig.add_shape(
            type='line', line=dict(dash='dash'),
            x0=0, x1=1, y0=0, y1=1
        )

        df = obj.data.dataframe
        classes = df['class'].unique()
        for class_ in classes:
            fpr = list(df.loc[df['class'] == class_]['fpr'])
            tpr = list(df.loc[df['class'] == class_]['tpr'])
            logger.debug(f"class_:{class_}/ fpr:{fpr} / tpr:{tpr}")
            fig.add_trace(plotly_module.Scatter(x=fpr, y=tpr, name=class_, mode='lines'))

        fig.update_layout(
            title=obj.label_fields["title"],
            xaxis_title='False Positive Rate',
            yaxis_title='True Positive Rate',
            yaxis=dict(scaleanchor="x", scaleratio=1),
            xaxis=dict(constrain='domain'),
        )
        fig.show()

    if isinstance(obj, Chart):
        if obj.chart_type == TYPE_CONFUSION_MATRIX:
            visualize_confusion_matrix(obj)
        elif obj.chart_type == TYPE_ROC_CURVE:
            visualize_roc_curve(obj)
        elif obj.chart_type == TYPE_HISTOGRAM:
            visualize_chart_histogram(obj)
        elif obj.chart_type == TYPE_BAR:
            visualize_chart_bar(obj)
        else:
            visualize_chart_common(obj)
    elif isinstance(obj, Table):
        visualize_table(obj)
    elif isinstance(obj, Image):
        visualize_image(obj)



import json

from assertpy import assert_that


class BoundingBoxes():
    def __init__(self, key: str, box_data: dict, class_labels: dict):
        self.key = key
        self.box_data = box_data
        self.class_labels = class_labels

        # validtaion
        self.validate()

    def validate(self):
        try:
            for data in self.box_data:
                assert_that(data).contains("position", "class_id", "scores", "caption")  # key값 검증
                assert_that(data['position']).is_instance_of(dict)
                assert_that(data['position']).contains("minX", "maxX", "minY", "maxY")
                assert_that(data['position']["minX"]).is_instance_of((float, int))
                assert_that(data['position']["maxX"]).is_instance_of((float, int))
                assert_that(data['position']["minY"]).is_instance_of((float, int))
                assert_that(data['position']["maxY"]).is_instance_of((float, int))
                assert_that(data['class_id']).is_instance_of(int)
                assert_that(data['scores']).contains("acc", "loss")
                assert_that(data['scores']["acc"]).is_instance_of((float, int))
                assert_that(data['scores']["loss"]).is_instance_of((float, int))
                if "unit" in data:
                    assert_that(data['unit']).is_instance_of(str)
                assert_that(data['caption']).is_instance_of(str)
        except AssertionError as e:
            raise Exception(f"BoundingBoxes Valition Error : {e.message}")

    def to_json(self):
        return json.dumps({
            "box_data": self.box_data,
            "class_labels": self.class_labels,
        })

    def to_dict(self):
        return {
            "box_data": self.box_data,
            "class_labels": self.class_labels,
        }


import multiprocessing
import os
import platform
import psutil
import pynvml

try:
    pynvml.nvmlInit()
except Exception:
    pass

def get_os() -> str:
    return platform.system()

def get_python_version() -> str:
    return platform.python_version()

def get_gpu() -> str:
    try:
        return pynvml.nvmlDeviceGetName(pynvml.nvmlDeviceGetHandleByIndex(0)).decode("utf-8")
    except Exception:
        return "no gpu"

def get_gpu_count() -> int:
    try:
        return pynvml.nvmlDeviceGetCount()
    except Exception:
        return 0

def get_cpu_count() -> int:
    return multiprocessing.cpu_count()

def get_hostname() -> str:
    return platform.node()

def get_system_cpu() -> float:
    return psutil.cpu_percent()

def get_system_disk() -> float:
    return psutil.disk_usage(os.path.sep).used / (2**30)

def get_system_memory() -> float:
    return psutil.virtual_memory().used / (2**30)

def get_system_proc_cpu_threads() -> int:
    pid = os.getpid() # get pid
    proc = psutil.Process(pid)
    return proc.num_threads()

def get_system_proc_memory_rss_mb() -> float:
    pid = os.getpid() # get pid
    proc = psutil.Process(pid)
    return proc.memory_info().rss / 1048576.0

def get_system_proc_memory_percent() -> float:
    pid = os.getpid() # get pid
    proc = psutil.Process(pid)
    return proc.memory_percent()

def get_system_memory_available_mb() -> float:
    return psutil.virtual_memory().available / 1048576.0


def is_notebook() -> bool:
    try:
        if 'google.colab' in str(get_ipython()):
            return True

        shell  = get_ipython().__class__.__name__
        if shell == 'ZMQInteractiveShell':
            return True   # Jupyter notebook or qtconsole
        elif shell == 'TerminalInteractiveShell':
            return False  # Terminal running IPython
        else:
            return False  # Other type (?)
    except NameError:
        return False      # Probably standard Python interpreter


def print_progress(iteration: int, total: int, prefix: str = 'Loading', suffix: str = '', bar_length: int = 30):
    # progressBar를 출력
    if total == 0:
        percent = 100
        filled_length = bar_length
    else:
        percent = ("{0:.1f}").format(100 * (iteration / float(total)))
        filled_length = int(round(bar_length * iteration / float(total)))
    bar = '█' * filled_length + '-' * (bar_length - filled_length)

    text = "%s |%s| [%s%%] %s" % (prefix, bar, percent, suffix)
    print('\r' + ' ' * 200, end='', flush=True) #jupyter에서 남아있는 부분을 삭제
    print('\r' + text, end='', flush=True)

    if percent == 100:
        print("\r\n")



from functools import wraps
def login_required(original_function):
    from deepdriver import logger
    ''' deepdriver 로그인 체크 '''
    @wraps(original_function)
    def wrapper(*args, **kwargs):
        from deepdriver.sdk.interface.interface import get_stub
        stub = get_stub()
        if not stub:
            logger.error("please log in first")
            raise Exception("Login required.")
        return original_function(*args, **kwargs)

    return wrapper


def init_required(original_function):
    from deepdriver import logger
    ''' deepdriver init() 체크 '''
    @wraps(original_function)
    def wrapper(*args, **kwargs):
        from deepdriver.sdk.data_types.run import get_run
        run = get_run()
        if not run:
            logger.error("please call deepdriver.init() first")
            raise Exception("Deepdriver not inited")
        return original_function(*args, **kwargs)

    return wrapper



from typing import Dict

from deepdriver import logger
from deepdriver.sdk.data_types.run import Run


class Config:

    def __init__(self, init_dict: Dict = None) -> None:
        pass

    # 하이퍼 파리미터와 같은 config 값 update
    def update(self) -> None:
        from deepdriver import interface
        from deepdriver.sdk.data_types.run import get_run
        # 서버로 config() 전송
        logger.debug("config update..")
        run: Run = get_run()
        return interface.update_config(run.run_id, run.team_name, run.exp_name, self.Items())


    def __setitem__(self, idx):
        pass

    def __getitem__(self, idx, value):
        pass

    def __contains__(self, name):
        pass

    def get(self, key: str):
        if key in self.__dict__:
            return self.__dict__[key]
        else:
            return None

    def Items(self):
        return list(self.__dict__.items())

    def keys(self) -> list:
        return list(self.__dict__.keys())

    def values(self) -> list:
        return list(self.__dict__.values())



import numpy as np
import pandas as pd
from assertpy import assert_that

from deepdriver.sdk.lib.lazyloader import LazyLoader
sklearn_matric = LazyLoader('deepdriver.metrics.roc_curve', globals(), 'sklearn.metrics')  #layzyload
#
# try:
#     from sklearn.metrics import roc_curve as sklearn_roc_curve
# except ImportError:
#     raise ImportError("sklearn package is required")

from deepdriver.sdk.chart.chart import Chart, TYPE_ROC_CURVE


def roc_curve(y_true: list, probs: list, class_names: list = None, title: str = None) -> Chart:
    # 필수값 체크 !!
    assert_that(y_true).is_not_none().is_type_of(list)
    assert_that(probs).is_not_none().is_type_of(list)


    y_true = np.array(y_true)
    y_probas = np.array(probs)
    classes = np.unique(y_true)

    fpr = dict()
    tpr = dict()
    indices_to_plot = np.where(np.isin(classes, classes))[0]
    for i in indices_to_plot:
        if class_names is not None and (
                isinstance(classes[i], int) or isinstance(classes[0], np.integer)
        ):
            class_label = class_names[classes[i]]
        else:
            class_label = classes[i]

        fpr[class_label], tpr[class_label], _ = sklearn_matric.roc_curve(
            y_true, y_probas[..., i], pos_label=classes[i]
        )

    df = pd.DataFrame(
        {
            "class": np.hstack([[k] * len(v) for k, v in fpr.items()]),
            "fpr": np.hstack(list(fpr.values())),
            "tpr": np.hstack(list(tpr.values())),
        }
    )
    df = df.round(3)
    # deepdriver data 프레임으로 변환
    from deepdriver import DataFrame
    dataframe = DataFrame(dataframe=df)
    fields = {"x": "fpr", "y": "tpr", "class": "class"}

    return Chart(chart_type=TYPE_ROC_CURVE, data=dataframe, data_fields=fields, label_fields={"title": title})



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from deepdriver.sdk.interface import grpc_interface_pb2 as deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2


class ResourceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.upload_log = channel.unary_unary(
                '/Resource/upload_log',
                request_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadLogRequest.SerializeToString,
                response_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadLogResponse.FromString,
                )
        self.upload_artifact = channel.unary_unary(
                '/Resource/upload_artifact',
                request_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadArtifactRequest.SerializeToString,
                response_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadArtifactResponse.FromString,
                )
        self.upload_file = channel.stream_unary(
                '/Resource/upload_file',
                request_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadFileRequest.SerializeToString,
                response_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadFileResponse.FromString,
                )
        self.use_artifact = channel.unary_unary(
                '/Resource/use_artifact',
                request_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UseArtifactRequest.SerializeToString,
                response_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UseArtifactResponse.FromString,
                )
        self.download_file = channel.unary_stream(
                '/Resource/download_file',
                request_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.DownloadFileRequest.SerializeToString,
                response_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.DownloadFileResponse.FromString,
                )
        self.update_config = channel.unary_unary(
                '/Resource/update_config',
                request_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UpdateConfigRequest.SerializeToString,
                response_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UpdateConfigResponse.FromString,
                )
        self.send_alert = channel.unary_unary(
                '/Resource/send_alert',
                request_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.SendAlertRequest.SerializeToString,
                response_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.SendAlertResponse.FromString,
                )


class ResourceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def upload_log(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def upload_artifact(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def upload_file(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def use_artifact(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def download_file(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def update_config(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def send_alert(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ResourceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'upload_log': grpc.unary_unary_rpc_method_handler(
                    servicer.upload_log,
                    request_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadLogRequest.FromString,
                    response_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadLogResponse.SerializeToString,
            ),
            'upload_artifact': grpc.unary_unary_rpc_method_handler(
                    servicer.upload_artifact,
                    request_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadArtifactRequest.FromString,
                    response_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadArtifactResponse.SerializeToString,
            ),
            'upload_file': grpc.stream_unary_rpc_method_handler(
                    servicer.upload_file,
                    request_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadFileRequest.FromString,
                    response_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadFileResponse.SerializeToString,
            ),
            'use_artifact': grpc.unary_unary_rpc_method_handler(
                    servicer.use_artifact,
                    request_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UseArtifactRequest.FromString,
                    response_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UseArtifactResponse.SerializeToString,
            ),
            'download_file': grpc.unary_stream_rpc_method_handler(
                    servicer.download_file,
                    request_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.DownloadFileRequest.FromString,
                    response_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.DownloadFileResponse.SerializeToString,
            ),
            'update_config': grpc.unary_unary_rpc_method_handler(
                    servicer.update_config,
                    request_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UpdateConfigRequest.FromString,
                    response_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UpdateConfigResponse.SerializeToString,
            ),
            'send_alert': grpc.unary_unary_rpc_method_handler(
                    servicer.send_alert,
                    request_deserializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.SendAlertRequest.FromString,
                    response_serializer=deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.SendAlertResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Resource', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Resource(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def upload_log(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Resource/upload_log',
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadLogRequest.SerializeToString,
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadLogResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def upload_artifact(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Resource/upload_artifact',
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadArtifactRequest.SerializeToString,
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadArtifactResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def upload_file(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/Resource/upload_file',
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadFileRequest.SerializeToString,
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UploadFileResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def use_artifact(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Resource/use_artifact',
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UseArtifactRequest.SerializeToString,
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UseArtifactResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def download_file(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/Resource/download_file',
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.DownloadFileRequest.SerializeToString,
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.DownloadFileResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def update_config(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Resource/update_config',
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UpdateConfigRequest.SerializeToString,
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.UpdateConfigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def send_alert(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Resource/send_alert',
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.SendAlertRequest.SerializeToString,
            deepdriver_dot_sdk_dot_interface_dot_grpc__interface__pb2.SendAlertResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import json
import logging
import requests
from  urllib.parse import ParseResult
from typing import Dict

from deepdriver.sdk import util
from deepdriver import logger
import deepdriver.version as version
URL_LOGIN = "/api/client/auth"
URL_CREATE = "/api/client/experiment/create"
URL_CREATE_HPO = "/api/client/experiment/create/hpo"
URL_GET_HPO = "/api/client/experiment/get/hpo"
URL_FINISH = "/api/client/run/finish"



jwt_key: str = None

def set_jwt_key(jwt_key_: str) -> None:
    global jwt_key
    jwt_key = jwt_key_

def get_jwt_key() -> str:
    global jwt_key
    return jwt_key

# 서버의 login api 를 호출하여 API key 값을 서버로 전송하고 결과로서 jwt key 를 받는다.
# 받은 jwt key 는 메모리에 저장해 두어서 추후 서버와 통신시 헤더에 실어서 보낼 수 있도록 한다.
def login(http_host: str , api_key: str) -> Dict:
    data = {
        "key": api_key
    }
    url = ParseResult(scheme="http",netloc=http_host, path=URL_LOGIN, params='', query='', fragment='').geturl()
    rsp = post(url, data)
    logger.debug("login() " + rsp["message"])
    if rsp["result"] == "success":
        set_jwt_key(rsp["token"])
    return rsp

def init(http_host: str ,exp_name: str="", team_name: str="", run_name: str="", config: Dict=None) -> Dict:
    # Interface.py의 init 함수 호출시,
    # config 값을 넘겨주어 rest api의 인자인 config의 _custom 필드에 key, value값을 각각 넣어주도록 한다
    _custom = [{"key": k, "value": v} for k, v in config.items()] if config else []

    # sysInfo값을 넘겨주어 rest api의 인자인 sysInfo필드에 넣어주도록 한다
    sys_info = {
        "os": util.get_os(),
        "python": util.get_python_version(),
        "gpu": util.get_gpu(),
        "gpuCount": util.get_gpu_count(),
        "cpuCount": util.get_cpu_count(),
        "hostname": util.get_hostname(),
    }

    # Interface.py의 init 함수 호출 후 응답받은 데이터로 부터 실험환경 이름과 팀이름, 실행이름, 실행ID, 대쉬보드URL을 가져와 Run 객체에 설정하고 반환한다.
    data = {
        "teamName": team_name,
        "expName": exp_name,
        "runName": run_name,
        "config": {"cliVer": version.__version__, "pythonVer": util.get_python_version(), "_custom": _custom},
        "sysInfo": sys_info,
        "createRun": "Y",
    }
    url = ParseResult(scheme="http",netloc=http_host, path=URL_CREATE, params='', query='', fragment='').geturl()
    rsp = post(url, data)
    logger.debug("init() response : " + rsp["message"])
    return rsp

def create_hpo(http_host:str, exp_name: str="", team_name: str="", hpo_config: Dict=None) -> Dict:

    data ={
        "teamName": team_name,
        "expName": exp_name,
        "config":  {"cliVer": version.__version__, "pythonVer": util.get_python_version(), "hpoConfig": hpo_config},
        "sysInfo": {
            "os": util.get_os(),
            "python": util.get_python_version(),
            "gpu": util.get_gpu(),
            "gpuCount": util.get_gpu_count(),
            "cpuCount": util.get_cpu_count(),
            "hostname": util.get_hostname(),
        },
    }
    url = ParseResult(scheme="http", netloc=http_host, path=URL_CREATE_HPO, params='', query='', fragment='').geturl()
    rsp = post(url, data)
    logger.debug("create_hpo() response : " + rsp["message"])
    return rsp


def get_hpo(http_host:str, exp_name: str="", team_name: str="") -> Dict:
    data = {
        "teamName": team_name,
        "expName": exp_name,
    }
    url = ParseResult(scheme="http", netloc=http_host, path=URL_GET_HPO, params='', query='', fragment='').geturl()
    rsp = post(url, data)
    logger.debug("get_hpo() response : " + rsp["message"])
    return rsp


def finish(http_host: str ,data: Dict) -> Dict:
    url = ParseResult(scheme="http",netloc=http_host, path=URL_FINISH, params='', query='', fragment='').geturl()
    rsp = post(url, data)
    logger.debug("finish() " + rsp["message"])
    return rsp

def post(url: str, data: Dict) -> Dict:
    logger.debug(f"REST[POST] to [{url}] : data=[{json.dumps(data)}], headers=[{get_headers()}")
    rsp = requests.post(url, headers=get_headers(), data=json.dumps(data))
    if rsp.status_code not in [200, 201]:
        logger.error(f"post({url}) failed({rsp.status_code})")
        raise Exception(f"request.post() failed({rsp.text if rsp.text else rsp.status_code})", rsp)
    return dict(json.loads(rsp.text))

def get_headers():
    jwt_key = get_jwt_key()
    headers = {
        "Content-Type": "application/json",
    }
    if jwt_key:
        headers["Authorization"] = f"Bearer {jwt_key}"
    return headers



from assertpy import assert_that

from deepdriver.sdk.chart.chart import Chart, TYPE_LINE
from deepdriver.sdk.data_types.dataFrame import DataFrame

def line(data: DataFrame, x: str, y: str, title: str=None) -> Chart:
    assert_that(data).is_not_none()
    assert_that(x).is_not_none()
    assert_that(y).is_not_none()

    return Chart(chart_type=TYPE_LINE, data=data, data_fields={"x": x, "y": y}, label_fields={"title": title})



import json
from assertpy import assert_that

LOG_TYPE_IMAGE = "image"
LOG_TYPE_MEDIA = "media"
LOG_TYPE_TABLE = "table"

class Media:

    def __init__(self, log_type: str=LOG_TYPE_MEDIA, path: str="") -> None:
        # 서버로 로깅되는 타입("media" 로 고정)
        self.log_type = log_type

        # 파일이 저장된 위치
        self.path = path

    # 서버로 전송될 파일 지정
    # 상속받는 자식 객체에서 공통적으로 사용할 함수
    def set_file(self) -> None:
        pass

    def to_json(self, key_name: str) -> str:
        assert_that(key_name).is_not_none()

        value_type = __class__.__name__
        return json.dumps({
            "log_type" : self.log_type,
            "hash" : "",
            "size" : 0,
            "path" : self.get_path(key_name, value_type),
        })

    # json으로 구성된 메타데이터 전송
    # 상속받는 자식 객체에서 공통적으로 사용할 함수
    def upload(self) -> None:
        pass

    # 실제 파일 서버로 전송
    # 상속받는 자식 객체에서 공통적으로 사용할 함수
    def upload_file(self) -> None:
        pass

    def get_path(self, key_name: str, value_type: str) -> str:
        return f"{key_name}.{value_type}.json"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deepdriver/sdk/interface/grpc_interface.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-deepdriver/sdk/interface/grpc_interface.proto\"\x93\x01\n\x10UploadLogRequest\x12\x16\n\x04item\x18\x01 \x03(\x0b\x32\x08.LogItem\x12\x16\n\x04step\x18\x02 \x01(\x0b\x32\x08.LogStep\x12\x15\n\x03run\x18\x03 \x01(\x0b\x32\x08.RunInfo\x12\x10\n\x08teamName\x18\x04 \x01(\t\x12\x0f\n\x07\x65xpName\x18\x05 \x01(\t\x12\x15\n\rauthorization\x18\x06 \x01(\t\"8\n\x11UploadLogResponse\x12#\n\nrsp_result\x18\x01 \x01(\x0b\x32\x0f.ResponseResult\"1\n\x0eResponseResult\x12\x0e\n\x06result\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\"\x94\x01\n\x07LogItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x17\n\rvalueAsString\x18\x02 \x01(\tH\x00\x12\x14\n\nvalueAsInt\x18\x03 \x01(\x05H\x00\x12\x16\n\x0cvalueAsFloat\x18\x04 \x01(\x02H\x00\x12\x15\n\x0bvalueAsLong\x18\x05 \x01(\x03H\x00\x12\x15\n\x0bvalueAsBool\x18\x06 \x01(\x08H\x00\x42\x07\n\x05value\"\x16\n\x07LogStep\x12\x0b\n\x03num\x18\x01 \x01(\x03\"\x19\n\x07RunInfo\x12\x0e\n\x06run_id\x18\x01 \x01(\x03\"Q\n\x15UploadArtifactRequest\x12!\n\x08\x61rtifact\x18\x01 \x01(\x0b\x32\x0f.ArtifactRecord\x12\x15\n\rauthorization\x18\x02 \x01(\t\"R\n\x16UploadArtifactResponse\x12\x13\n\x0b\x61rtifact_id\x18\x01 \x01(\x03\x12#\n\nrsp_result\x18\x02 \x01(\x0b\x32\x0f.ResponseResult\"\xd2\x01\n\x0e\x41rtifactRecord\x12\x0e\n\x06run_id\x18\x01 \x01(\x03\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x0e\n\x06\x64igest\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x12\n\nversioning\x18\x06 \x01(\t\x12\x10\n\x08metadata\x18\x07 \x01(\t\x12\x10\n\x08teamName\x18\x08 \x01(\t\x12\x0f\n\x07\x65xpName\x18\t \x01(\t\x12&\n\x0e\x61rtifact_entry\x18\n \x03(\x0b\x32\x0e.ArtifactEntry\"\x98\x01\n\rArtifactEntry\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06\x64igest\x18\x02 \x01(\t\x12\x0c\n\x04size\x18\x03 \x01(\x03\x12\x0e\n\x06status\x18\x04 \x01(\t\x12\r\n\x05lfsYN\x18\x05 \x01(\t\x12\x0f\n\x07repoTag\x18\x06 \x01(\t\x12\x0b\n\x03key\x18\x07 \x01(\t\x12\x0c\n\x04type\x18\x08 \x01(\t\x12\x10\n\x08metadata\x18\t \x01(\t\"\xad\x02\n\x11UploadFileRequest\x12\x12\n\nuploadType\x18\x01 \x01(\t\x12\x19\n\x04\x66ile\x18\x02 \x01(\x0b\x32\x0b.FileRecord\x12\x13\n\x0b\x61rtifact_id\x18\x03 \x01(\x03\x12\x0e\n\x06run_id\x18\x04 \x01(\x03\x12\x14\n\x0c\x61rtifactName\x18\x05 \x01(\t\x12\x14\n\x0c\x61rtifactType\x18\x06 \x01(\t\x12\x0e\n\x06\x64igest\x18\x07 \x01(\t\x12\x14\n\x0clast_file_yn\x18\x08 \x01(\t\x12\x15\n\rauthorization\x18\t \x01(\t\x12\x10\n\x08teamName\x18\n \x01(\t\x12\x0f\n\x07\x65xpName\x18\x0b \x01(\t\x12\x0f\n\x07runName\x18\x0c \x01(\t\x12\'\n\x0ftotal_file_info\x18\r \x01(\x0b\x32\x0e.TotalFileInfo\"9\n\x12UploadFileResponse\x12#\n\nrsp_result\x18\x01 \x01(\x0b\x32\x0f.ResponseResult\"%\n\nFileRecord\x12\x17\n\x04\x66ile\x18\x01 \x01(\x0b\x32\t.FileItem\"H\n\x08\x46ileItem\x12\x1d\n\x08\x66ilepath\x18\x01 \x01(\x0b\x32\t.FilePathH\x00\x12\x12\n\x08\x63ontents\x18\x02 \x01(\x0cH\x00\x42\t\n\x07request\"+\n\x08\x46ilePath\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x11\n\troot_path\x18\x02 \x01(\t\">\n\rTotalFileInfo\x12\x0e\n\x06\x64igest\x18\x01 \x01(\t\x12\x1d\n\x05\x65ntry\x18\x02 \x03(\x0b\x32\x0e.ArtifactEntry\"\xa4\x01\n\x12UseArtifactRequest\x12\x15\n\rartifact_name\x18\x01 \x01(\t\x12\x15\n\rartifact_type\x18\x02 \x01(\t\x12\x14\n\x0c\x61rtifact_tag\x18\x03 \x01(\t\x12\x11\n\tteam_name\x18\x04 \x01(\t\x12\x10\n\x08\x65xp_name\x18\x05 \x01(\t\x12\x0e\n\x06run_id\x18\x06 \x01(\x03\x12\x15\n\rauthorization\x18\x07 \x01(\t\"r\n\x13UseArtifactResponse\x12\x13\n\x0b\x61rtifact_id\x18\x01 \x01(\x03\x12#\n\nrsp_result\x18\x02 \x01(\x0b\x32\x0f.ResponseResult\x12!\n\x08\x61rtifact\x18\x03 \x01(\x0b\x32\x0f.ArtifactRecord\"\xd2\x01\n\x13\x44ownloadFileRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x13\n\x0b\x61rtifact_id\x18\x02 \x01(\x03\x12\x14\n\x0c\x61rtifactName\x18\x03 \x01(\t\x12\x14\n\x0c\x61rtifactType\x18\x04 \x01(\t\x12\x10\n\x08teamName\x18\x05 \x01(\t\x12\x0f\n\x07\x65xpName\x18\x06 \x01(\t\x12\x12\n\nversioning\x18\x07 \x01(\t\x12\r\n\x05lfsYN\x18\x08 \x01(\t\x12\x0f\n\x07repoTag\x18\t \x01(\t\x12\x15\n\rauthorization\x18\n \x01(\t\"]\n\x14\x44ownloadFileResponse\x12#\n\nrsp_result\x18\x01 \x01(\x0b\x32\x0f.ResponseResult\x12\x10\n\x08\x63ontents\x18\x02 \x01(\x0c\x12\x0e\n\x06\x64igest\x18\x03 \x01(\t\"\x97\x01\n\nConfigItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x17\n\rvalueAsString\x18\x02 \x01(\tH\x00\x12\x14\n\nvalueAsInt\x18\x03 \x01(\x05H\x00\x12\x16\n\x0cvalueAsFloat\x18\x04 \x01(\x02H\x00\x12\x15\n\x0bvalueAsLong\x18\x05 \x01(\x03H\x00\x12\x15\n\x0bvalueAsBool\x18\x06 \x01(\x08H\x00\x42\x07\n\x05value\"y\n\x13UpdateConfigRequest\x12\r\n\x05runId\x18\x01 \x01(\x03\x12\x19\n\x04item\x18\x02 \x03(\x0b\x32\x0b.ConfigItem\x12\x10\n\x08teamName\x18\x03 \x01(\t\x12\x0f\n\x07\x65xpName\x18\x04 \x01(\t\x12\x15\n\rauthorization\x18\x05 \x01(\t\";\n\x14UpdateConfigResponse\x12#\n\nrsp_result\x18\x01 \x01(\x0b\x32\x0f.ResponseResult\"Z\n\x10SendAlertRequest\x12\r\n\x05runId\x18\x01 \x01(\x03\x12\x10\n\x08teamName\x18\x02 \x01(\t\x12\x0f\n\x07\x65xpName\x18\x03 \x01(\t\x12\x14\n\x0c\x61lertMessage\x18\x04 \x01(\t\"8\n\x11SendAlertResponse\x12#\n\nrsp_result\x18\x01 \x01(\x0b\x32\x0f.ResponseResult2\xb9\x03\n\x08Resource\x12\x35\n\nupload_log\x12\x11.UploadLogRequest\x1a\x12.UploadLogResponse\"\x00\x12\x44\n\x0fupload_artifact\x12\x16.UploadArtifactRequest\x1a\x17.UploadArtifactResponse\"\x00\x12:\n\x0bupload_file\x12\x12.UploadFileRequest\x1a\x13.UploadFileResponse\"\x00(\x01\x12;\n\x0cuse_artifact\x12\x13.UseArtifactRequest\x1a\x14.UseArtifactResponse\"\x00\x12@\n\rdownload_file\x12\x14.DownloadFileRequest\x1a\x15.DownloadFileResponse\"\x00\x30\x01\x12>\n\rupdate_config\x12\x14.UpdateConfigRequest\x1a\x15.UpdateConfigResponse\"\x00\x12\x35\n\nsend_alert\x12\x11.SendAlertRequest\x1a\x12.SendAlertResponse\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'deepdriver.sdk.interface.grpc_interface_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _UPLOADLOGREQUEST._serialized_start=50
  _UPLOADLOGREQUEST._serialized_end=197
  _UPLOADLOGRESPONSE._serialized_start=199
  _UPLOADLOGRESPONSE._serialized_end=255
  _RESPONSERESULT._serialized_start=257
  _RESPONSERESULT._serialized_end=306
  _LOGITEM._serialized_start=309
  _LOGITEM._serialized_end=457
  _LOGSTEP._serialized_start=459
  _LOGSTEP._serialized_end=481
  _RUNINFO._serialized_start=483
  _RUNINFO._serialized_end=508
  _UPLOADARTIFACTREQUEST._serialized_start=510
  _UPLOADARTIFACTREQUEST._serialized_end=591
  _UPLOADARTIFACTRESPONSE._serialized_start=593
  _UPLOADARTIFACTRESPONSE._serialized_end=675
  _ARTIFACTRECORD._serialized_start=678
  _ARTIFACTRECORD._serialized_end=888
  _ARTIFACTENTRY._serialized_start=891
  _ARTIFACTENTRY._serialized_end=1043
  _UPLOADFILEREQUEST._serialized_start=1046
  _UPLOADFILEREQUEST._serialized_end=1347
  _UPLOADFILERESPONSE._serialized_start=1349
  _UPLOADFILERESPONSE._serialized_end=1406
  _FILERECORD._serialized_start=1408
  _FILERECORD._serialized_end=1445
  _FILEITEM._serialized_start=1447
  _FILEITEM._serialized_end=1519
  _FILEPATH._serialized_start=1521
  _FILEPATH._serialized_end=1564
  _TOTALFILEINFO._serialized_start=1566
  _TOTALFILEINFO._serialized_end=1628
  _USEARTIFACTREQUEST._serialized_start=1631
  _USEARTIFACTREQUEST._serialized_end=1795
  _USEARTIFACTRESPONSE._serialized_start=1797
  _USEARTIFACTRESPONSE._serialized_end=1911
  _DOWNLOADFILEREQUEST._serialized_start=1914
  _DOWNLOADFILEREQUEST._serialized_end=2124
  _DOWNLOADFILERESPONSE._serialized_start=2126
  _DOWNLOADFILERESPONSE._serialized_end=2219
  _CONFIGITEM._serialized_start=2222
  _CONFIGITEM._serialized_end=2373
  _UPDATECONFIGREQUEST._serialized_start=2375
  _UPDATECONFIGREQUEST._serialized_end=2496
  _UPDATECONFIGRESPONSE._serialized_start=2498
  _UPDATECONFIGRESPONSE._serialized_end=2557
  _SENDALERTREQUEST._serialized_start=2559
  _SENDALERTREQUEST._serialized_end=2649
  _SENDALERTRESPONSE._serialized_start=2651
  _SENDALERTRESPONSE._serialized_end=2707
  _RESOURCE._serialized_start=2710
  _RESOURCE._serialized_end=3151
# @@protoc_insertion_point(module_scope)



import hashlib
import json
import os
from pathlib import Path
from typing import Dict

from assertpy import assert_that
from typing import Dict, Tuple, TYPE_CHECKING
from deepdriver import logger
from deepdriver.sdk.data_types.dataFrame import DataFrame
from deepdriver.sdk.data_types.run import Run
from deepdriver.sdk.interface import interface

TYPE_HISTOGRAM = "histogram"
TYPE_LINE = "line"
TYPE_SCATTER = "scatter"
TYPE_BAR = "bar"
TYPE_CONFUSION_MATRIX = "confusion_matrix"
TYPE_ROC_CURVE = "roc_curve"


class Chart:

    def __init__(self, data: DataFrame, chart_type: str, data_fields: Dict, label_fields: Dict = None) -> None:
        assert_that(data).is_not_none()
        assert_that(chart_type).is_not_none()
        assert_that(data_fields).is_not_none()

        self.data = data
        self.log_type = "chart"
        self.chart_type = chart_type
        self.data_fields = data_fields
        self.label_fields = label_fields if label_fields else {}

    @classmethod
    def from_file(cls, path: str):
        assert_that(str).is_not_none()
        with open(path) as f:
            data = json.load(f)
        df = DataFrame(columns=data['data']['columns'], data=data['data']['data'])
        return cls(chart_type=data['chart_type'], data=df, data_fields=data['data_fields'], label_fields=data['label_fields'])

    def to_dict(self):
        return {
            "data": self.data.to_dict(),
            "log_type": self.log_type,
            "chart_type": self.chart_type,
            "data_fields": self.data_fields,
            "label_fields": self.label_fields,
        }

    def to_json(self, key_name: str) -> str:
        assert_that(key_name).is_not_none()


        return json.dumps({
            "log_type": self.log_type,
            "chart_type": self.chart_type,
            "data_fields": self.data_fields,
            "label_fields": self.label_fields,
            "path": self.get_path(key_name),
        })

    def upload_file(self, run: Run, key_name: str) -> None:
        local_path = self.get_local_path(run.run_id, key_name)
        digest, size = self.file_dump(local_path)

        # 서버로 파일 전송
        root_path = self.get_root_path(run.run_id)
        path = self.get_path(key_name)
        logger.debug(f"file upload[chart] : local_path=[{local_path}], root_path=[{root_path}], path=[{path}]")

        interface.upload_file(upload_type="RUN", local_path=local_path, root_path=root_path, path=path,
                              run_id=run.run_id, artifact_id=0, last_file_yn="Y", teamName=run.team_name,
                              expName=run.exp_name, run_name=run.run_name,
                              artifact_name="", artifact_type="", artifact_digest="", entry_digest=digest,
                              entry_list=[], file_index=0)


    def file_dump(self, path: str) -> Tuple[str, str]:
        Path(os.path.dirname(path)).mkdir(parents=True, exist_ok=True)

        with open(path, "w") as f:
            json.dump(self.to_dict(), f)

        with open(path, "rb") as f:
            digest = hashlib.md5(f.read()).hexdigest()
            f.seek(0, os.SEEK_END)
            size = f.tell()
        return digest, size


    def get_path(self, key_name: str) -> str:
        return f"{key_name}.CHART.json"

    def get_local_dir_path(self, run_id: int) -> str:
        return os.path.join(".", "deepdriver", "run", self.get_root_path(run_id))

    def get_root_path(self, run_id: int) -> str:
        return os.path.join(str(run_id), "chart")

    def get_local_path(self, run_id: int, key_name: str) -> str:
        return os.path.join(self.get_local_dir_path(str(run_id)), self.get_path(key_name))



import deepdriver
from deepdriver.sdk.lib.lazyloader import LazyLoader
torch = LazyLoader('deepdriver.lazy.torch', globals(), 'torch')

from deepdriver import logger

LOG_TRACK_COUNT, LOG_TRACK_THRESHOLD = range(2)


# 함수
def log_track_init(log_freq: int):
    """create tracking structure used by log_track_update"""
    l = [0] * 2
    l[LOG_TRACK_THRESHOLD] = log_freq
    return l


def log_track_update(log_track: int) -> bool:
    """count (log_track[0]) up to threshold (log_track[1]), reset count (log_track[0]) and return true when reached"""
    log_track[LOG_TRACK_COUNT] += 1
    if log_track[LOG_TRACK_COUNT] < log_track[LOG_TRACK_THRESHOLD]:
        return False
    log_track[LOG_TRACK_COUNT] = 0
    return True


#
class TorchLog:
    def add_log_gradients_hook(self, module, name, prefix, log_freq: int = 0):
        prefix = prefix + name

        if not hasattr(module, "_deepdriver_hook_names"):
            module._deepdriver_hook_names = []

        for name, parameter in module.named_parameters():
            print(parameter.requires_grad)
            if parameter.requires_grad:
                log_track_grad = log_track_init(log_freq)
                module._deepdriver_hook_names.append("gradients/" + prefix + name)
                self._hook_variable_gradient_stats(
                    parameter, "gradients/" + prefix + name, log_track_grad
                )

    def add_log_parameters_hook(
            self,
            module: "torch.nn.Module",
            name: str = "",
            prefix: str = "",
            log_freq: int = 0,
    ) -> None:
        """This instruments hooks into the pytorch module
        log parameters after a forward pass
        log_freq - log gradients/parameters every N batches
        """
        # if name is not None:
        prefix = prefix + name

        if not hasattr(module, "_deepdriver_hook_names"):
            module._deepdriver_hook_names = []

        def parameter_log_hook(module, input_, output, log_track):
            if not log_track_update(log_track):
                return
            for name, parameter in module.named_parameters():
                # for pytorch 0.3 Variables
                if isinstance(parameter, torch.autograd.Variable):
                    data = parameter.data
                else:
                    data = parameter
                self.log_tensor_stats(data.cpu(), "parameters/" + prefix + name)

        log_track_params = log_track_init(log_freq)
        try:
            hook = module.register_forward_hook(
                lambda mod, inp, outp: parameter_log_hook(
                    mod, inp, outp, log_track_params
                )
            )
            # self._hook_handles["parameters/" + prefix] = hook
            module._deepdriver_hook_names.append("parameters/" + prefix)
        except RuntimeError as e:
            print(
                f"Trying to register forward_hook failed ({e}) - skipping parameter tracking."
            )

    def _hook_variable_gradient_stats(self, var, name, log_track):
        """Logs a Variable's gradient's distribution statistics next time backward()
        is called on it.
        """

        def _callback(grad, log_track):
            if not log_track_update(log_track):
                return
            self.log_tensor_stats(grad.data, name)

        handle = var.register_hook(lambda grad: _callback(grad, log_track))
        # self._hook_handles[name] = handle
        return handle

    def log_tensor_stats(self, tensor, name):

        """Add distribution statistics on a tensor's elements to the current History entry"""
        # TODO Handle the case of duplicate names.

        if isinstance(tensor, tuple) or isinstance(tensor, list):
            while (isinstance(tensor, tuple) or isinstance(tensor, list)) and (
                    isinstance(tensor[0], tuple) or isinstance(tensor[0], list)
            ):
                tensor = [item for sublist in tensor for item in sublist]
            tensor = torch.cat([t.reshape(-1) for t in tensor])

        # checking for inheritance from _TensorBase didn't work for some reason
        if not hasattr(tensor, "shape"):
            cls = type(tensor)
            raise TypeError(f"Expected Tensor, not {cls.__module__}.{cls.__name__}")

        # HalfTensors on cpu do not support view(), upconvert to 32bit
        if isinstance(tensor, torch.HalfTensor):
            tensor = tensor.clone().type(torch.FloatTensor).detach()

        # Sparse tensors have a bunch of implicit zeros. In order to histo them correctly,
        # we have to count them up and add them to the histo ourselves.
        sparse_zeros = None
        if tensor.is_sparse:
            # Have to call this on a sparse tensor before most other ops.
            tensor = tensor.cpu().coalesce().clone().detach()

            backing_values = tensor._values()
            non_zero_values = backing_values.numel()
            all_values = tensor.numel()
            sparse_zeros = all_values - non_zero_values
            tensor = backing_values

        flat = tensor.reshape(-1)

        # For pytorch 0.3 we use unoptimized numpy histograms (detach is new in 0.4)
        if not hasattr(flat, "detach"):
            tensor = flat.cpu().clone().numpy()
            print("detach")
            deepdriver.log({name: deepdriver.histogram(seq=tensor)})
            return
        histogram = deepdriver.histogram(seq=tensor)
        # deepdriver.visualize(histogram)  # 코드 반영시엔 주석 처리할것!
        deepdriver.log({name: histogram})


torch_log: TorchLog = None


def set_torch_log(torch_log_: TorchLog) -> None:
    global torch_log
    torch_log = torch_log_


def get_torch_log() -> TorchLog:
    global torch_log
    if not torch_log:
        logger.info("TorchLog is Initialized")
        torch_log = TorchLog()
    return torch_log



from __future__ import annotations

import json
import time
from assertpy import assert_that
from typing import Dict, Tuple, TYPE_CHECKING



from deepdriver import logger
from deepdriver.sdk import util
from deepdriver.sdk.interface import interface
from deepdriver.sdk.interface.grpc_interface_pb2 import *
if TYPE_CHECKING:
    from deepdriver.sdk.artifact import Artifact

run: Run = None

def set_run(run_: Run) -> None:
    global run
    run = run_

def get_run() -> Run:
    global run
    return run

class Run:

    def __init__(self, team_name: str, exp_name: str, run_name: str, run_id: int=0, run_url: str="") -> None:
        assert_that(team_name).is_not_none()
        assert_that(exp_name).is_not_none()
        assert_that(run_name).is_not_none()

        self.team_name = team_name
        self.exp_name = exp_name
        self.run_name = run_name
        self.run_id = run_id
        self.run_url = run_url
        self.run_start_time = time.time()

        # 로그 스텝(Run.log로 기록 될 때마다 1씩 증가)
        self.log_step = 0

        self.lastData = {}

    # Dictionary 형태의 데이터를 기반으로 서버로 전송하는 함수
    def log(self, data: Dict) -> bool:
        assert_that(data).is_not_none()

        # item에 key값이 “timestamp” 이고 value가 현재시각 (time.time())인 item도 추가
        # item에 로그의 key value값의 쌍이 들어가도록 item을 추가ex) log({“loss” :0.03, “acc” :0.97}) -> key가 loss 이고 value가 0.03인 item과 key가 acc이고 value가 0.97인 item이 추가됨


        from deepdriver.sdk.data_types.image import Image
        from deepdriver.sdk.data_types.table import Table
        from deepdriver.sdk.chart.chart import Chart

        for key, value in data.items():
            # TODO: Image, Chart등 다른 데이터 형식도 추가해야함
            if isinstance(value, Table):
                data[key] = value.to_json(key)
                logger.debug(f"Table을 str(json)로 변환 변환 : {data[key]}")
                value.upload_file(self, key)
            elif isinstance(value, Image):
                data[key] = value.to_json(key)
                logger.debug(f"Image를 str(json)로 변환 변환 : {data[key]}")
                value.upload_file(self, key)
            elif isinstance(value, Chart):
                data[key] = value.to_json(key)
                logger.debug(f"Chart를 str(json)로 변환 변환 : {data[key]}")
                value.upload_file(self, key)
            elif isinstance(value, list):
                if all(isinstance(n, Image) for n in value):    #Image 객체 배열일 경우
                        list_dict = []
                        for index, n in enumerate(value):
                            list_dict.append(n.to_dict(key_name=key, is_list=True, index=index))
                            n.upload_file(self, key, is_list=True, index=index)
                        data[key] = json.dumps(list_dict)


                        logger.debug(f"Image 객체(배열)을 str(json)로 변환 변환 : {data[key]}")




        data["timestamp"] = time.time()

        # Item의 key, value에 각각 하기의 정보를 넣는다
        data["system.cpu"] = util.get_system_cpu()
        data["system.disk"] = util.get_system_disk()
        data["system.memory"] = util.get_system_memory()
        data["system.proc.cpu.threads"] = util.get_system_proc_cpu_threads()
        data["system.proc.memory.rssMB"] = util.get_system_proc_memory_rss_mb()
        data["system.proc.memory.percent"] = util.get_system_proc_memory_percent()
        data["system.proc.memory.availableMB"] = util.get_system_memory_available_mb()

        self.log_step += 1
        succeeded = interface.upload_log(self.run_id, self.team_name, self.exp_name, self.log_step, data)

        # 마지막으로 사용된 data 를 보관
        for key, value in data.items():
            self.lastData[key] = value

        return succeeded

    # Artifact 객체를 기반으로 서버로 전송하는 함수
    def upload_artifact(self, artifact: Artifact) -> bool:
        assert_that(artifact).is_not_none()
        return artifact.upload()

    def get_artifact(self, name: str, type: str, tag: str="", team_name: str="", exp_name: str="") -> Tuple[
        int, ArtifactRecord]:
        assert_that(name).is_not_none()
        assert_that(type).is_not_none()

        # 아티팩트의 이름과 타입을 서버로 전송하여 아티팩트 사용기록을 남기고 아티팩트 관련 정보를 받아온다
        # Tag 미지정시 서버에서 최신 버전의 아티팩트의 정보를 받아온다
        # 팀이름과 실험환경 미지정시 run에 설정된 정보로 팀과 실험환경을 판단하여 하위 아티팩트를 받아온다
        # 서버로부터 응답으로 아티팩트 ID 및 ArtifactRecord정보를 전달 받아서 Artifact 객체에 설정한다
        return interface.use_artifact(name, type, tag, team_name, exp_name, self.run_id)

    def finish(self) -> bool:
        _custom = [{"key": k, "value": v} for k, v in self.lastData.items()]
        data = {
            "runId": self.run_id,
            "summary": {
                "_runtime": time.time()-self.run_start_time,
                "_step": self.log_step,
                "_custom": _custom,
            },
        }
        resp = interface.finish(data)
        return resp["result"] == "success"


    def alert(self, alert_msg:str):
        return interface.send_alert(self.run_id, self.team_name, self.exp_name, alert_msg)






from typing import Union

from assertpy import assert_that

from deepdriver.sdk.chart.chart import Chart, TYPE_HISTOGRAM
from deepdriver.sdk.data_types.dataFrame import DataFrame
import numpy as np
import pandas as pd
def histogram(data: DataFrame = None, x: str ="", y: str = "", title: str = "", seq: Union[np.ndarray, list]=None) -> Chart:
    # assert_that(data).is_not_none()
    # assert_that(x).is_not_none()
    assert_that([data, seq]).is_not_none()

    if not data:
        pd_dataframe = pd.DataFrame(np.hstack(seq).tolist(), columns=["x"])
        data = DataFrame(dataframe=pd_dataframe)
        x = "x"

    return Chart(chart_type=TYPE_HISTOGRAM, data=data, data_fields={"x": x, "y": y}, label_fields={"title": title})



from typing import Literal, Optional
from deepdriver.sdk.lib.lazyloader import LazyLoader
torch = LazyLoader('deepdriver.lazy.torch', globals(), 'torch')

from deepdriver.intergration.torch.torch import get_torch_log

_global_watch_idx = 0


def watch(models, log: Optional[Literal["gradients", "parameters", "all"]] = "gradients", log_freq: int = 1000,
          log_graph: bool = False):
    if log != "gradients" and log != "parameters" and log != "all":
        raise ValueError("log must be one of 'gradients', 'parameters', 'all'")

    if not isinstance(models, (tuple, list)):
        models = (models,)

    global _global_watch_idx
    idx = _global_watch_idx

    prefix = ""
    for local_idx, model in enumerate(models):
        global_idx = idx + local_idx
        _global_watch_idx += 1

        if global_idx > 0:
            prefix = "graph_%i" % global_idx

        if not isinstance(model, torch.nn.Module):
            raise ValueError(
                "Expected a pytorch model (torch.nn.Module). Received "
                + str(type(model))
            )

        if log == "gradients" or log == "all":
            get_torch_log().add_log_gradients_hook(model, name="", prefix=prefix, log_freq=log_freq)

        if log == "parameters" or log == "all":
            get_torch_log().add_log_parameters_hook(model, name="", prefix=prefix, log_freq=log_freq)




import setuptools
from distutils.util import convert_path

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

ns = {}
ver_path = convert_path('deepdriver/version.py')
with open(ver_path) as ver_file:
    exec(ver_file.read(), ns)

setuptools.setup(
    name="deepdriver",
    version=ns['__version__'],
    author="bokchi",
    author_email="molamola.bokchi@gmail.com",
    description="deepdriver experiments",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bokchi.com",
    project_urls={
        "bokchi git hub": "https://github.com/molabokchi",
    },
    install_requires=[
        "wheel",
        "assertpy",
        "grpcio",
        "grpcio-tools",
        "numpy",
        "pandas",
        "Pillow",
        "plotly",
        "psutil",
        "pynvml",
        "requests",
        "optuna",
        "psycopg2"
    ],

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"":"."},
    packages=["deepdriver","deepdriver.sdk","deepdriver.intergration","deepdriver.intergration.keras","deepdriver.sdk.chart","deepdriver.sdk.data_types","deepdriver.sdk.interface","deepdriver.sdk.lib","deepdriver.intergration.torch",""],
    python_requires=">=3.6"
)


from __future__ import annotations
from __future__ import annotations

import json
import os
import sys
from typing import Dict, List, Tuple, TYPE_CHECKING, Union
from urllib.parse import urlsplit, urlparse

import grpc

from deepdriver import logger
from deepdriver.sdk import util
from deepdriver.sdk.interface import http_interface
from deepdriver.sdk.interface.grpc_interface_pb2 import *
from deepdriver.sdk.interface.grpc_interface_pb2_grpc import ResourceStub

if TYPE_CHECKING:
    from deepdriver.sdk.artifact import Artifact

stub: ResourceStub = None

def set_stub(stub_: ResourceStub) -> None:
    global stub
    stub = stub_


def get_stub() -> ResourceStub:
    global stub
    return stub

grpc_host : str = "127.0.0.1:19051"

def set_grpc_host(grpc_host_: str) -> None:
    global grpc_host
    grpc_host = grpc_host_

def get_grpc_host() -> str:
    global grpc_host
    return grpc_host

http_host = "127.0.0.1:9011"

def set_http_host(http_host_: str) -> None:
    global http_host
    if http_host_.upper().startswith("HTTP"):
        http_host  = urlparse(http_host_).netloc
    else:
        http_host = http_host_


def get_http_host() -> str:
    global http_host
    return http_host

def get_http_host_ip() -> str:
    global http_host
    return urlsplit('//' + http_host).hostname


def update_config(run_id: int, team_name: str, exp_name: str, config_item_list: list(Tuple)):
    def makeItem(item):
        key, value = item
        if isinstance(value, int):
            if value > 2 ** 32 - 1:  # check type long
                configItem = ConfigItem(key=key, valueAsLong=value)
            else:
                configItem = ConfigItem(key=key, valueAsInt=value)
        elif isinstance(value, str):
            configItem = ConfigItem(key=key, valueAsString=value)
        elif isinstance(value, float):
            configItem = ConfigItem(key=key, valueAsFloat=value)
        elif isinstance(value, bool):
            configItem = ConfigItem(key=key, valueAsBool=value)
        elif isinstance(value, dict) or isinstance(value, list):
            configItem = ConfigItem(key=key, valueAsString=json.dumps(value))
        return configItem

    # client 정보 추가
    import deepdriver.version as version
    config_item_list.append(('cliVer', version.__version__))
    config_item_list.append(('pythonVer', util.get_python_version()))


    item = map(makeItem, config_item_list)
    rsp: UpdateConfigResponse = get_stub().update_config(UpdateConfigRequest(
        runId=run_id,
        item=item,
        teamName=team_name,
        expName=exp_name,
        authorization=f"Bearer {http_interface.get_jwt_key()}",
    ))

    return rsp.rsp_result.result == "success"


def send_alert(run_id: int, team_name: str, exp_name: str, alert_msg: str):
    rsp: SendAlertResponse = get_stub().send_alert(SendAlertRequest(
        runId=run_id,
        teamName=team_name,
        expName=exp_name,
        alertMessage=alert_msg,
    ))

    return rsp.rsp_result.result == "success"

def upload_log(run_id: int, team_name: str, exp_name: str, log_step: int, item_dict: Dict[str, Union[int, str, float, bool]]) -> bool:
    item_list = []

    def makeItem(item):
        key, value = item
        if isinstance(value, int):
            if value > 2 ** 32 - 1:  # check type long
                logItem = LogItem(key=key, valueAsLong=value)
            else:
                logItem = LogItem(key=key, valueAsInt=value)
        elif isinstance(value, str):
            logItem = LogItem(key=key, valueAsString=value)
        elif isinstance(value, float):
            logItem = LogItem(key=key, valueAsFloat=value)
        elif isinstance(value, bool):
            logItem = LogItem(key=key, valueAsBool=value)
        else:
            raise Exception("value는 int, str, float, bool 타입만 가능합니다.")
        return logItem

    item = map(makeItem, item_dict.items())

    # item = [LogItem(key=key, value=str(value)) for key, value in item_dict.items()]
    rsp: UploadLogResponse = get_stub().upload_log(UploadLogRequest(
        item=item,
        step=LogStep(num=log_step),
        run=RunInfo(run_id=run_id),
        teamName=team_name,
        expName=exp_name,
        authorization=f"Bearer {http_interface.get_jwt_key()}",
    ))
    return rsp.rsp_result.result == "success"


CHUNK_SIZE = 1024 * 1024  # 1MB


def load_file(upload_type: str, local_path: str, root_path: str, path: str, run_id: int, artifact_id: int,
              last_file_yn: str, team_name: str, exp_name: str, run_name: str, artifact_name: str, artifact_type: str,
              artifact_digest: str, entry_digest: str, entry_list: List[ArtifactEntry],
              file_index: int) -> UploadFileRequest:
    if sys.platform.startswith('win32'):
        # 윈도우인 경우 Path Seperator를 리눅스 형식으로 강제로 변경함
        root_path = root_path.replace("\\", "/")
        path = path.replace("\\", "/")
    logger.debug(f"root_path: [{root_path}], path:[{path}] ")
    yield UploadFileRequest(
        uploadType=upload_type,
        file=FileRecord(file=FileItem(
            filepath=FilePath(
                path=path,
                root_path=root_path,
            )
        )),
        artifact_id=artifact_id,
        run_id=run_id,
        artifactName=artifact_name,
        artifactType=artifact_type,
        digest=entry_digest,
        last_file_yn=last_file_yn,
        authorization=f"Bearer {http_interface.get_jwt_key()}",
        teamName=team_name,
        expName=exp_name,
        runName=run_name,
        total_file_info=TotalFileInfo(
            digest=artifact_digest,
            entry=entry_list,
        ),
    )
    with open(local_path, "rb") as file:
        uploaded_size = 0
        total_size = os.stat(local_path).st_size
        while True:

            chunk_bytes = file.read(CHUNK_SIZE)
            uploaded_size += len(chunk_bytes)
            util.print_progress(uploaded_size, total_size, f'Uploading: [{local_path}]',
                                f"[{file_index + 1}/{1 if len(entry_list) == 0 else len(entry_list)}")
            if len(chunk_bytes) == 0:  # Reached EOF
                util.print_progress(total_size, total_size, f'Uploading: [{local_path}]',
                                    f"[{file_index + 1}/{1 if len(entry_list) == 0 else len(entry_list)}]")  # 100%
                return
            yield UploadFileRequest(
                file=FileRecord(file=FileItem(
                    contents=chunk_bytes
                )),
                artifact_id=artifact_id,
                run_id=run_id,
                artifactName=artifact_name,
                artifactType=artifact_type,
                digest=entry_digest,
                last_file_yn=last_file_yn,
                authorization=f"Bearer {http_interface.get_jwt_key()}",
                teamName=team_name,
                expName=exp_name,
                runName=run_name,
                total_file_info=TotalFileInfo(
                    digest=artifact_digest,
                    entry=entry_list,
                ),
            )


def upload_file(upload_type: str, local_path: str, root_path: str, path: str, run_id: int, artifact_id: int,
                last_file_yn: str, teamName: str, expName: str, run_name: str, artifact_name: str, artifact_type: str,
                artifact_digest: str, entry_digest: str, entry_list: List[ArtifactEntry], file_index: int) -> bool:
    logger.debug(
        f"upload_file() : {locals()}")

    rsp: UploadFileResponse = get_stub().upload_file(
        load_file(upload_type, local_path, root_path, path, run_id, artifact_id, last_file_yn, teamName, expName,
                  run_name, artifact_name, artifact_type, artifact_digest, entry_digest, entry_list, file_index))
    return rsp.rsp_result.result == "success"


def save_file(file_path: str, file_index: int, total_file_count: int, file_size: int, rsps) -> None:
    logger.debug(f"save_file() : file_path:[{file_path}]")

    with open(file_path, "wb") as file:
        downloaded_size = 0
        for rsp in rsps:
            chunk = bytes(rsp.contents)
            downloaded_size += len(chunk)
            util.print_progress(downloaded_size, file_size, f'Downloading: [{file_path}]',
                                f"[{file_index + 1}/{total_file_count}]")
            if len(chunk) == 0:
                raise Exception("empty chunk")
            file.write(chunk)
    util.print_progress(file_size, file_size, f'Downloading: [{file_path}]',
                        f"[{file_index + 1}/{total_file_count}]")  # 100%
    return None


def download_file(path: str, artifact_id: int, local_path: str, team_name: str, exp_name: str, artifact_name: str,
                  artifact_type: str, versioning: str, lfs_yn: str, repo_tag: str, file_index: int,
                  total_file_count: int,
                  file_size: int):
    # TODO 응답에 digest 값을 받아 추후 각 파일에 대한 정합성 체크시 활용한다.

    logger.debug(
        f"download_file() : {locals()}")

    save_file(local_path, file_index, total_file_count, file_size, get_stub().download_file(DownloadFileRequest(
        path=path,
        artifact_id=artifact_id,
        authorization=f"Bearer {http_interface.get_jwt_key()}",
        teamName=team_name,
        expName=exp_name,
        artifactName=artifact_name,
        artifactType=artifact_type,
        versioning=versioning,
        repoTag=repo_tag,
        lfsYN=lfs_yn,

    )))


def upload_artifact(run_id: int, artifact: Artifact, artifact_digest: str, teamName: str, expName: str,
                    entry_list: List[ArtifactEntry]) -> int:
    logger.debug(f"""
            upload_artifact(): ( 
                "run_id":"{run_id}",
                "type":"{artifact.type}",
                "name":"{artifact.name}",
                "digest":"{artifact_digest}",
                "description":"{artifact.desc}",
                "versioning":"{artifact.versioning}",
                "metadata":"{json.dumps(artifact.meta_data)}",
                "teamName":"{teamName}",
                "expName":"{expName}",
                "artifact_entry":"{entry_list}",
            )""")

    rsp: UploadArtifactResponse = get_stub().upload_artifact(UploadArtifactRequest(
        artifact=ArtifactRecord(
            run_id=run_id,
            type=artifact.type,
            name=artifact.name,
            digest=artifact_digest,
            description=artifact.desc,
            versioning=artifact.versioning,
            metadata=json.dumps(artifact.meta_data),
            teamName=teamName,
            expName=expName,
            artifact_entry=entry_list,
        ),
        authorization=f"Bearer {http_interface.get_jwt_key()}",
    ))
    return rsp.artifact_id if rsp.rsp_result.result == "success" else None


def use_artifact(name: str, type: str, tag: str, team_name: str, exp_name: str, run_id: int) -> Tuple[int,
                                                                                                      ArtifactRecord]:
    logger.debug(f"""
        use_artifact(): ( 
            "artifact_name":"{name}",
            "artifact_type":"{type}",
            "artifact_tag":"{tag}",
            "team_name":"{team_name}",
            "exp_name":"{exp_name}",
            "run_id":"{run_id}",
            "authorization": "Bearer {http_interface.get_jwt_key()}"
        )""")

    try:
        rsp: UseArtifactResponse = get_stub().use_artifact(UseArtifactRequest(
            artifact_name=name,
            artifact_type=type,
            artifact_tag=tag,
            team_name=team_name,
            exp_name=exp_name,
            run_id=run_id,
            authorization=f"Bearer {http_interface.get_jwt_key()}",
        ))
    except grpc.RpcError as e:
        logger.exception("Network Error")
        return "fail", "network error", -1, None
    return rsp.rsp_result.result, rsp.rsp_result.message, rsp.artifact_id, rsp.artifact


def download_artifact():
    pass


def login(key: str) -> bool:
    rsp = http_interface.login(get_http_host(), key)
    if rsp["result"] == "success":

        grpc_url = get_grpc_host()

        if grpc_url is None:
            grpc_url = rsp["grpcHost"]
        # grpc stub 생성
        channel = grpc.insecure_channel(grpc_url)
        stub = ResourceStub(channel)
        set_stub(stub)
    return rsp["result"] == "success"


def init(exp_name: str = "", team_name: str = "", run_name: str = "", config: Dict = None) -> Dict:
    return http_interface.init(get_http_host(), exp_name, team_name, run_name, config)

def create_hpo(exp_name: str = "", team_name: str = "", hpo_config: Dict = None) -> Dict:
    return http_interface.create_hpo(get_http_host(), exp_name, team_name, hpo_config)

def get_hpo(exp_name: str = "", team_name: str = "") -> Dict:
    return http_interface.get_hpo(get_http_host(), exp_name, team_name)

def finish(data: Dict) -> Dict:
    return http_interface.finish(get_http_host(), data)






import hashlib
import json
import os
from pathlib import Path

from assertpy import assert_that
from typing import Dict, Tuple, TYPE_CHECKING
from deepdriver import logger
from deepdriver.sdk.data_types.dataFrame import DataFrame
from deepdriver.sdk.data_types.media import LOG_TYPE_TABLE, Media
from deepdriver.sdk.data_types.run import Run
from deepdriver.sdk.interface import interface


class Table(Media):

    def __init__(self, data: DataFrame) -> None:
        super().__init__(log_type=LOG_TYPE_TABLE)

        assert_that(data).is_not_none()
        self.data = data

    @classmethod
    def from_file(cls, path: str):
        assert_that(str).is_not_none()
        with open(path) as f:
            data = json.load(f)
        return cls(data=DataFrame(columns=data['data']['columns'], data=data['data']['data']))

    def __str__(self):
        return self.data.dataframe.__str__()

    def __repr__(self):
        # return "1"
        return self.data.dataframe.__repr__()

    def _repr_html_(self):
        # return "1"
        return self.data.dataframe._repr_html_()

    def to_dict(self):
        return {
            "data": self.data.to_dict(),
        }

    def to_json(self, key_name: str) -> str:
        assert_that(key_name).is_not_none()

        return json.dumps({
            "log_type": self.log_type,
            "path": self.get_path(key_name),
            "cols": len(self.data.dataframe.columns),
            "rows": len(self.data.dataframe),
        })



    # 실제 파일 서버로 전송
    # Run.log에 Image 객체가 기록된 경우 Image 의 값을 올리기 위한 함수
    # table내용을 json으로 변환하여 파일로 저장후 전송
    def upload_file(self, run: Run, key_name: str) -> None:
        # Table의 data인 deepdriver.dataframe으로부터 column과 data를 가져와서 json형태의 파일로 저장
        local_path = self.get_local_path(run.run_id, key_name)
        digest, size = self.file_dump(local_path)

        # 저장한 파일을 Interface.py의 upoad_file을 호출하여 전송
        root_path = self.get_root_path(run.run_id)
        path = self.get_path(key_name)
        logger.debug(f"file upload[table] : local_path=[{local_path}], root_path=[{root_path}], path=[{path}]")

        interface.upload_file(upload_type="RUN", local_path=local_path, root_path=root_path, path=path,
                              run_id=run.run_id, artifact_id=0, last_file_yn="Y", teamName=run.team_name,
                              expName=run.exp_name, run_name=run.run_name, artifact_name="", artifact_type="",
                              artifact_digest="", entry_digest=digest, entry_list=[], file_index=0)

    def file_dump(self, path: str) -> Tuple[str, str]:
        Path(os.path.dirname(path)).mkdir(parents=True, exist_ok=True)

        with open(path, "w") as f:
            json.dump(self.to_dict(), f)

        with open(path, "rb") as f:
            digest = hashlib.md5(f.read()).hexdigest()
            f.seek(0, os.SEEK_END)
            size = f.tell()
        return digest, size

    def get_root_path(self, run_id: int) -> str:
        return os.path.join(str(run_id), "media")

    def get_local_dir_path(self, run_id: int) -> str:
        return os.path.join(".", "deepdriver", "run", self.get_root_path(run_id))

    def get_path(self, key_name: str) -> str:
        return f"{key_name}.TABLE.json"

    def get_local_path(self, run_id: int, key_name: str) -> str:
        return os.path.join(self.get_local_dir_path(str(run_id)), self.get_path(key_name))
